SystemOrganization addCategory: #'OpenGL-NameManager'!

Object subclass: #NullTerminatedStringArray
	instanceVariableNames: 'pointers externalStrings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenGL-NameManager'!

!NullTerminatedStringArray commentStamp: 'jcg 8/7/2005 18:47' prior: 0!
NullTerminatedStringArray is used for passing data to FFI functions that require a null-terminated array of null-terminated strings.  For example, this class is suitable for the 'char* argv[]' argument of the 'main()' function in C programs.  Another example is the arguments passed to the Cg compiler in the context of an OpenGL/Cg application.  The first example does not require a null-terminated array, because the number of strings is stored in the 'argc' argument.  However, the second example requires both the strings and the array to be null-terminated.

'pointers' stores a null-terminated list of pointers to the null-terminated strings stored in 'externalStrings'.!

!NullTerminatedStringArray class methodsFor: 'instance creation' stamp: 'jcg 9/5/2003 02:06'!
withStrings: stringCollection

	^ self new strings: stringCollection! !

!NullTerminatedStringArray methodsFor: 'initialization' stamp: 'jcg 3/30/2005 07:54'!
free

	(pointers isNil and: [externalStrings isNil]) ifTrue:
		[^ self "Nothing to free."].

	externalStrings do: [:string | string free].
	pointers := nil.
	externalStrings := nil.! !

!NullTerminatedStringArray methodsFor: 'accessing' stamp: 'jcg 9/5/2003 03:00'!
pointers
	"Answer a null-terminated array of pointers (an ExternalData) to null-terminated strings."

	^ pointers! !

!NullTerminatedStringArray methodsFor: 'testing' stamp: 'jcg 3/30/2005 07:49'!
strings
	"Get Smalltalk Strings from byte data."

	^ externalStrings collect: 
		[:string | 
		(ExternalData fromHandle: string type: ExternalType char asPointerType) fromCString]! !

!NullTerminatedStringArray methodsFor: 'testing' stamp: 'jcg 3/30/2005 07:52'!
strings2
	"Get Smalltalk Strings from byte data.  This way is longer, but makes sure that 'pointers' is also correct."

	^ (1 to: externalStrings size) collect: 
		[:ind | 
		(ExternalData 
			fromHandle: (pointers getHandle pointerAt: (ind-1)*4+1)
			type: ExternalType char asPointerType) fromCString]! !

!NullTerminatedStringArray methodsFor: 'initialization' stamp: 'jcg 3/30/2005 07:46'!
strings: stringCollection
	| extern pointerBytes |

	(pointers notNil or: [externalStrings notNil]) ifTrue:
		[self error: 'Pointers have not been freed.'].

	"Allocate space for each string individually, and copy the bytes over."
	externalStrings := stringCollection collect: 
		[:string |
		extern := ExternalAddress allocate: string size + 1.
		1 to: string size do: [:ind | extern byteAt: ind put: (string byteAt: ind)].
		extern byteAt: string size + 1 put: 0.
		extern].

	"Create null-terminated array of pointers to strings."
	pointerBytes := ByteArray new: 4*(stringCollection size + 1).
	pointers := ExternalData fromHandle: pointerBytes type: ExternalType void asPointerType.
	externalStrings withIndexDo: 
		[:string :ind |
		pointerBytes pointerAt: (ind-1)*4 + 1 put: string].
! !
